package cn.com.qmth.examcloud.web.security;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import cn.com.qmth.examcloud.api.commons.CloudService;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ServletUtil;
import cn.com.qmth.examcloud.web.support.StatusResponse;

/**
 * spring cloud 请求接入
 *
 * @author WANGWEI
 * @date 2018年5月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public final class SpringCloudInterceptor implements HandlerInterceptor {

	private static final ExamCloudLog LOG = ExamCloudLogFactory
			.getLog(SpringCloudInterceptor.class);

	/**
	 * 接口日志
	 */
	protected static final ExamCloudLog INTERFACE_LOG = ExamCloudLogFactory
			.getLog("INTERFACE_LOGGER");

	/**
	 * redis client
	 */
	private RedisClient redisClient;

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {
		LOG.debug("preHandle... ...");

		Class<?> ctrClass = (Class<?>) request.getAttribute("$ctrClass");
		if (!CloudService.class.isAssignableFrom(ctrClass)) {
			return true;
		}

		Long timestamp = redisClient.get("$_RMI_:" + ThreadLocalUtil.getTraceID(), Long.class);

		if (null == timestamp) {
			response.setStatus(HttpStatus.REQUEST_TIMEOUT.value());
			ServletUtil.returnJson(new StatusResponse("408", "Request Timeout"), response);
			return false;
		}

		return true;
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
			ModelAndView modelAndView) throws Exception {
		LOG.debug("postHandle... ...");
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {
		LOG.debug("afterCompletion... ...");
	}

	public void setRedisClient(RedisClient redisClient) {
		this.redisClient = redisClient;
	}

}
